@@ -6,6 +6,7 @@ DFM is a PyTorch Lightning module that inherits from BaseFactorModel.
 
 # Standard library imports
 from dataclasses import dataclass
+from datetime import datetime
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, Union
 
@@ -235,10 +236,7 @@ class DFM(BaseFactorModel):
         loss : torch.Tensor
             Negative log-likelihood (to minimize)
         """
-        # CRITICAL FIX: Skip additional EM iterations if fit_em() already completed training
-        # This prevents double training that causes log-likelihood deterioration.
-        # fit_em() runs in on_train_start() and completes the full EM algorithm.
-        # Lightning's training loop should not run additional EM iterations after that.
+        # Skip if fit_em() already completed training
         if self.training_state is not None:
             # fit_em() has already run - use its results instead of doing more EM iterations
             loglik = self.training_state.loglik
@@ -362,17 +360,12 @@ class DFM(BaseFactorModel):
         device = X.device
         dtype = X.dtype
         
-        # CRITICAL: Impute missing data BEFORE EM loop
-        # Initialize parameters (this performs imputation internally)
+        # Initialize parameters and impute missing data
         self.initialize_from_data(X)
         
-        # After initialization, get the imputed data for EM loop
-        # initialize_from_data uses rem_nans_spline_torch internally, but we need
-        # to apply the same imputation to the data used in EM loop
+        # Impute missing data for EM loop
         from dfm_python.utils.data import rem_nans_spline_torch
-        opt_nan = {'method': self.nan_method, 'k': self.nan_k}
-        X_imputed, _ = rem_nans_spline_torch(X, method=opt_nan.get('method', 2), k=opt_nan.get('k', 3))
-        # Remove any remaining NaN/inf
+        X_imputed, _ = rem_nans_spline_torch(X, method=self.nan_method, k=self.nan_k)
         X_imputed = torch.where(torch.isfinite(X_imputed), X_imputed, torch.tensor(0.0, device=device, dtype=dtype))
         
         # Store imputed data
@@ -388,6 +381,10 @@ class DFM(BaseFactorModel):
         num_iter = 0
         converged = False
         
+        # Track parameter stability for diagnostics
+        previous_A_norm = None
+        previous_C_norm = None
+        
         # EM loop
         while num_iter < self.max_iter and not converged:
             # Create EM step parameters
@@ -416,23 +413,15 @@ class DFM(BaseFactorModel):
             # Perform EM step - use self.em(...) instead of em_step(...)
             C_new, R_new, A_new, Q_new, Z_0_new, V_0_new, loglik = self.em(em_params)
             
-            # Check for NaN in updated parameters (early stopping for numerical instability)
-            nan_detected = False
-            if torch.any(torch.isnan(C_new)) or torch.any(torch.isnan(A_new)) or \
-               torch.any(torch.isnan(Q_new)) or torch.any(torch.isnan(R_new)) or \
-               torch.any(torch.isnan(Z_0_new)) or torch.any(torch.isnan(V_0_new)) or \
-               (isinstance(loglik, float) and (np.isnan(loglik) or np.isinf(loglik))):
-                nan_detected = True
-                _logger.error(
-                    f"EM algorithm: NaN/Inf detected in parameters at iteration {num_iter + 1}. "
-                    f"Stopping early to prevent further numerical instability. "
-                    f"C NaN: {torch.any(torch.isnan(C_new)).item()}, "
-                    f"A NaN: {torch.any(torch.isnan(A_new)).item()}, "
-                    f"Q NaN: {torch.any(torch.isnan(Q_new)).item()}, "
-                    f"R NaN: {torch.any(torch.isnan(R_new)).item()}, "
-                    f"loglik: {loglik}"
-                )
-                # Don't update parameters if NaN detected - keep previous values
+            # Check for NaN in parameters (early stopping)
+            has_nan = (
+                torch.any(torch.isnan(C_new)) or torch.any(torch.isnan(A_new)) or
+                torch.any(torch.isnan(Q_new)) or torch.any(torch.isnan(R_new)) or
+                torch.any(torch.isnan(Z_0_new)) or torch.any(torch.isnan(V_0_new)) or
+                (isinstance(loglik, float) and (np.isnan(loglik) or np.isinf(loglik)))
+            )
+            if has_nan:
+                _logger.error(f"EM algorithm: NaN/Inf detected at iteration {num_iter + 1}, stopping early")
                 break
             
             # Update parameters
@@ -444,6 +433,26 @@ class DFM(BaseFactorModel):
                 self.Z_0.data = Z_0_new
                 self.V_0.data = V_0_new
             
+            # Track parameter stability
+            try:
+                A_norm = torch.linalg.norm(A_new).item()
+                C_norm = torch.linalg.norm(C_new).item()
+                
+                if previous_A_norm is not None and previous_C_norm is not None and num_iter > 5:
+                    A_change = abs(A_norm - previous_A_norm) / max(previous_A_norm, 1e-10)
+                    C_change = abs(C_norm - previous_C_norm) / max(previous_C_norm, 1e-10)
+                    param_change = max(A_change, C_change)
+                    
+                    if param_change < 1e-6:
+                        _logger.debug(f"EM: Parameters stable (change={param_change:.2e}) at iter {num_iter + 1}")
+                    elif param_change > 10.0:
+                        _logger.warning(f"EM: Parameters changing significantly (change={param_change:.2e}) at iter {num_iter + 1}")
+                
+                previous_A_norm = A_norm
+                previous_C_norm = C_norm
+            except (RuntimeError, ValueError):
+                pass
+            
             # Track best log-likelihood and parameters (for early stopping on divergence)
             if loglik > best_loglik:
                 best_loglik = loglik
@@ -457,9 +466,8 @@ class DFM(BaseFactorModel):
                     'V_0': self.V_0.data.clone()
                 }
             
-            # Check for significant log-likelihood deterioration (divergence detection)
-            # Stop early if log-likelihood worsens by more than 1000 from best (indicates divergence)
-            if num_iter > 10 and best_loglik != float('-inf') and loglik < best_loglik - 1000:
+            # Check for log-likelihood deterioration
+            if num_iter > 10 and loglik < best_loglik - 1000:
                 _logger.warning(
                     f"EM algorithm: Log-likelihood deteriorated significantly from best value. "
                     f"Best: {best_loglik:.4f}, Current: {loglik:.4f}, Deterioration: {best_loglik - loglik:.4f}. "
@@ -479,13 +487,10 @@ class DFM(BaseFactorModel):
                 loglik = best_loglik
                 break
             
-            # Check convergence - use self.em.check_convergence() instead of em_converged()
+            # Check convergence
             if num_iter > 2:
                 converged, change = self.em.check_convergence(
-                    loglik,
-                    previous_loglik,
-                    self.threshold,
-                    verbose=(num_iter % 10 == 0)
+                    loglik, previous_loglik, self.threshold, verbose=(num_iter % 10 == 0)
                 )
             else:
                 change = abs(loglik - previous_loglik) if previous_loglik != float('-inf') else 0.0
@@ -494,18 +499,15 @@ class DFM(BaseFactorModel):
             num_iter += 1
             
             # Log metrics using Lightning (enables TensorBoard, WandB, etc.)
-            # Note: on_step=False because fit_em may be called from on_train_start
+            # on_step=False because fit_em may be called from on_train_start
             self.log('train/loglik', loglik, on_step=False, on_epoch=True)
             self.log('train/em_iteration', float(num_iter), on_step=False, on_epoch=True)
             self.log('train/loglik_change', change, on_step=False, on_epoch=True)
             
-            # Log progress more frequently for visibility
+            # Log progress
             if num_iter % 5 == 0 or num_iter == 1:
-                print(f"[EM {num_iter:4d}/{self.max_iter:4d}] Log-likelihood: {loglik:12.6f} | Change: {change:10.6e} {'✓' if converged else ''}")
-                _logger.info(
-                    f"EM iteration {num_iter}/{self.max_iter}: "
-                    f"loglik={loglik:.4f}, change={change:.2e}"
-                )
+                status = " ✓" if converged else ""
+                _logger.info(f"EM iteration {num_iter}/{self.max_iter}: loglik={loglik:.4f}, change={change:.2e}{status}")
         
         # Store final state
         self.training_state = DFMTrainingState(
@@ -520,12 +522,13 @@ class DFM(BaseFactorModel):
             converged=converged
         )
         
-        # Final status message
+        # Final status
         if converged:
-            print(f"\n✓ EM algorithm converged after {num_iter} iterations (loglik: {loglik:.6f})")
+            print(f"\n✓ EM converged after {num_iter} iterations (loglik: {loglik:.6f})")
         else:
-            print(f"\n⚠ EM algorithm stopped after {num_iter} iterations (loglik: {loglik:.6f}, change: {change:.2e})")
-        _logger.info(f"EM training completed: converged={converged}, iterations={num_iter}, final_loglik={loglik:.6f}")
+            print(f"\n⚠ EM stopped after {num_iter} iterations (loglik: {loglik:.6f}, change: {change:.2e})")
+        
+        _logger.info(f"EM training completed: converged={converged}, iterations={num_iter}, loglik={loglik:.6f}")
         
         return self.training_state
     
@@ -672,6 +675,7 @@ class DFM(BaseFactorModel):
         self,
         horizon: Optional[int] = None,
         *,
+        history: Optional[int] = None,
         return_series: bool = True,
         return_factors: bool = True
     ) -> Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
@@ -685,6 +689,11 @@ class DFM(BaseFactorModel):
         horizon : int, optional
             Number of periods ahead to forecast. If None, defaults to 1 year
             of periods based on clock frequency.
+        history : int, optional
+            Number of historical periods to use for Kalman filter update before prediction.
+            If None, uses full history (default). If specified (e.g., 60), uses only the most
+            recent N periods for efficiency. Initial state (Z_0, V_0) is always estimated from
+            full history (including any new data beyond training period).
         return_series : bool, optional
             Whether to return forecasted series (default: True)
         return_factors : bool, optional
@@ -699,82 +708,133 @@ class DFM(BaseFactorModel):
                 X_forecast (horizon x N)
             If only return_factors is True:
                 Z_forecast (horizon x m)
+            
+        Notes
+        -----
+        When history is specified, the method uses only the most recent N periods for
+        Kalman filter update, improving computational efficiency. The initial state
+        (Z_0, V_0) is always estimated from full history (including any new data beyond
+        training period), ensuring accuracy while maintaining efficiency.
         """
         if self.training_state is None:
-            error_msg = self._format_error_message(
-                operation="prediction",
-                reason="model has not been trained yet",
-                guidance="Please call trainer.fit(model, data_module) first"
+            raise ValueError(
+                f"{self.__class__.__name__} prediction failed: model has not been trained yet. "
+                f"Please call trainer.fit(model, data_module) first"
             )
-            raise ValueError(error_msg)
         
-        # Convert training state to result format for prediction
+        # Get result (only call get_result() if _result is None)
         if not hasattr(self, '_result') or self._result is None:
             self._result = self.get_result()
         
         result = self._result
         
-        # Default horizon: 1 year of periods based on clock frequency
+        if not hasattr(result, 'Z') or result.Z is None:
+            raise ValueError(
+                "DFM prediction failed: result.Z is not available. "
+                "This may indicate the model was not properly trained or result object is corrupted."
+            )
+        
+        # Compute default horizon
         if horizon is None:
             from ..config.utils import get_periods_per_year
             from ..utils.helpers import get_clock_frequency
             clock = get_clock_frequency(self.config, 'm')
             horizon = get_periods_per_year(clock)
         
+        # Validate horizon
         if horizon <= 0:
-            raise ValueError(
-                f"DFM prediction failed: horizon must be a positive integer, got {horizon}. "
-                f"Please provide a positive integer value for the forecast horizon."
-            )
+            raise ValueError(f"horizon must be positive, got {horizon}")
         
         # Extract model parameters
         A = result.A
         C = result.C
         Wx = result.Wx
         Mx = result.Mx
-        Z_last = result.Z[-1, :]
+        p = getattr(result, 'p', 1)  # VAR order, default to 1 for DFM
         
-        # Validate that model is properly trained (Z_last should not contain NaN)
+        # Update factor state with history if specified
+        if history is not None and history > 0:
+            Z_last_updated = self._update_factor_state_with_history(
+                history=history,
+                result=result,
+                kalman_filter=getattr(self, 'kalman', None)
+            )
+            if Z_last_updated is not None:
+                Z_last = Z_last_updated
+            else:
+                # Fallback to training state if update failed
+                Z_last = result.Z[-1, :]
+        else:
+            # Use training state (default behavior)
+            Z_last = result.Z[-1, :]
+        
+        # Validate factor state
         if np.any(np.isnan(Z_last)):
             nan_count = np.sum(np.isnan(Z_last))
             nan_ratio = nan_count / len(Z_last)
             raise ValueError(
-                f"DFM prediction failed: {nan_count}/{len(Z_last)} factors contain NaN values ({nan_ratio:.1%}). "
-                f"This usually indicates the model did not converge during training. "
-                f"Try increasing max_iter, checking data quality, or removing series with high missing data."
+                f"DFM prediction failed: {nan_count}/{len(Z_last)} factors contain NaN ({nan_ratio:.1%}). "
+                f"Model may not have converged. Try increasing max_iter or checking data quality."
             )
         
         # Validate parameters are finite
         if np.any(~np.isfinite(A)) or np.any(~np.isfinite(C)):
             raise ValueError(
-                "DFM prediction failed: model parameters (A or C) contain NaN or Inf values. "
-                "This indicates the model did not train successfully. "
-                "Please check training convergence and data quality."
+                "DFM prediction failed: model parameters (A or C) contain NaN/Inf. "
+                "Check training convergence and data quality."
             )
         
-        # Deterministic forecast: iteratively apply transition matrix A
-        Z_forecast = np.zeros((horizon, Z_last.shape[0]))
-        Z_forecast[0, :] = A @ Z_last
-        for h in range(1, horizon):
-            Z_forecast[h, :] = A @ Z_forecast[h - 1, :]
+        # Forecast factors using VAR dynamics (common helper)
+        Z_prev = result.Z[-2, :] if result.Z.shape[0] >= 2 and p == 2 else None
+        Z_forecast = self._forecast_var_factors(
+            Z_last=Z_last,
+            A=A,
+            p=p,
+            horizon=horizon,
+            Z_prev=Z_prev
+        )
         
-        # Transform factors to observed series: X = Z @ C^T, then denormalize
-        X_forecast_std = Z_forecast @ C.T
-        X_forecast = X_forecast_std * Wx + Mx
+        # Transform factors to observations (common helper)
+        X_forecast = self._transform_factors_to_observations(
+            Z_forecast=Z_forecast,
+            C=C,
+            Wx=Wx,
+            Mx=Mx
+        )
         
         # Validate forecast results are finite
         if np.any(~np.isfinite(X_forecast)):
             nan_count = np.sum(~np.isfinite(X_forecast))
             raise ValueError(
                 f"DFM prediction failed: produced {nan_count} NaN/Inf values in forecast. "
-                f"This may indicate numerical instability. "
+                f"Possible numerical instability. "
                 f"Please check model parameters and data quality."
             )
+        
+        # Validate forecast values are within reasonable bounds
+        if Wx is not None and Mx is not None and len(Wx) > 0 and len(Mx) > 0:
+            # Check each series individually
+            extreme_threshold_std = 50.0  # Flag if forecast is > 50 std devs from mean
+            for i in range(X_forecast.shape[1] if X_forecast.ndim > 1 else 1):
+                if i < len(Wx) and i < len(Mx) and Wx[i] > 0:
+                    series_forecast = X_forecast[:, i] if X_forecast.ndim > 1 else X_forecast
+                    series_mean = Mx[i]
+                    series_std = Wx[i]
+                    # Calculate how many standard deviations each forecast is from the mean
+                    abs_deviations = np.abs(series_forecast - series_mean) / series_std
+                    max_deviation = np.max(abs_deviations) if len(abs_deviations) > 0 else 0.0
+                    if max_deviation > extreme_threshold_std:
+                        extreme_count = np.sum(abs_deviations > extreme_threshold_std)
+                        _logger.warning(
+                            f"DFM prediction: Extreme forecast for series {i} "
+                            f"(max deviation: {max_deviation:.1f} std devs). "
+                            f"Possible numerical instability."
+                        )
         if return_factors and np.any(~np.isfinite(Z_forecast)):
             nan_count = np.sum(~np.isfinite(Z_forecast))
             raise ValueError(
                 f"DFM prediction failed: produced {nan_count} NaN/Inf values in factor forecast. "
-                f"This may indicate numerical instability in factor dynamics. "
+                f"Possible numerical instability in factor dynamics. "
                 f"Please check model parameters and training convergence."
             )
         
@@ -804,5 +864,7 @@ class DFM(BaseFactorModel):
         super().reset()
         return self
     
+    # nowcast() is inherited from BaseFactorModel
+    
 
 
