@@ -5,13 +5,35 @@ to keep the main file under 1000 lines.
 """
 
 import numpy as np
-from typing import Optional, Tuple, Dict, Any
+from typing import Optional, Tuple, Dict, Any, TYPE_CHECKING
 import warnings
 from ..logger import get_logger
 
 _logger = get_logger(__name__)
 
-# Numerical stability constants (matching state_space.py)
+# PyTorch imports (optional, for DDFM utilities)
+if TYPE_CHECKING:
+    import torch
+    import torch.nn as nn
+else:
+    torch = None
+    nn = None
+
+try:
+    import torch
+    import torch.nn as nn
+    import torch.nn.functional as F
+    _has_torch = True
+except ImportError:
+    _has_torch = False
+    if not TYPE_CHECKING:
+        torch = None
+        nn = None
+    F = None
+
+# Numerical stability constants for NumPy operations
+# Note: Similar constants exist in ssm/utils.py (DEFAULT_MIN_EIGENVAL, DEFAULT_MIN_DIAGONAL_VARIANCE)
+# for PyTorch operations. They have the same values but are kept separate for context clarity.
 MIN_EIGENVAL_CLEAN = 1e-8
 MIN_DIAGONAL_VARIANCE = 1e-6
 DEFAULT_VARIANCE_FALLBACK = 1.0
@@ -159,20 +181,9 @@ def _safe_determinant(M: np.ndarray, use_logdet: bool = True) -> float:
     return 0.0
 
 
-# Re-export for backward compatibility
-# These functions are used by other modules
-def _compute_principal_components(cov_matrix: np.ndarray, n_components: int,
-                                   block_idx: Optional[int] = None) -> Tuple[np.ndarray, np.ndarray]:
-    """Compute top principal components via eigendecomposition with fallbacks.
-    
-    This function is a wrapper around encoder.pca.compute_principal_components
-    for backward compatibility.
-    """
-    from ..encoder.pca import compute_principal_components
-    return compute_principal_components(cov_matrix, n_components, block_idx=block_idx)
 
 
-def _compute_covariance_safe(data: np.ndarray, rowvar: bool = True, 
+def _compute_cov_safe(data: np.ndarray, rowvar: bool = True, 
                               pairwise_complete: bool = False,
                               min_eigenval: float = 1e-8,
                               fallback_to_identity: bool = True) -> np.ndarray:
@@ -184,7 +195,7 @@ def _compute_covariance_safe(data: np.ndarray, rowvar: bool = True,
     
     # Handle 1D case
     if data.ndim == 1:
-        var_val = _compute_variance_safe(data, ddof=0, min_variance=MIN_VARIANCE_COVARIANCE, 
+        var_val = _compute_var_safe(data, ddof=0, min_variance=MIN_VARIANCE_COVARIANCE, 
                                          default_variance=DEFAULT_VARIANCE_FALLBACK)
         return np.array([[var_val]])
     
@@ -194,7 +205,7 @@ def _compute_covariance_safe(data: np.ndarray, rowvar: bool = True,
     # Handle single variable case
     if n_vars == 1:
         series_data = data.flatten()
-        var_val = _compute_variance_safe(series_data, ddof=0, min_variance=MIN_VARIANCE_COVARIANCE,
+        var_val = _compute_var_safe(series_data, ddof=0, min_variance=MIN_VARIANCE_COVARIANCE,
                                          default_variance=DEFAULT_VARIANCE_FALLBACK)
         return np.array([[var_val]])
     
@@ -286,7 +297,7 @@ def _compute_covariance_safe(data: np.ndarray, rowvar: bool = True,
         raise
 
 
-def _compute_variance_safe(data: np.ndarray, ddof: int = 0, 
+def _compute_var_safe(data: np.ndarray, ddof: int = 0, 
                            min_variance: float = MIN_VARIANCE_COVARIANCE,
                            default_variance: float = DEFAULT_VARIANCE_FALLBACK) -> float:
     """Compute variance safely with robust error handling."""
@@ -309,7 +320,7 @@ def _compute_variance_safe(data: np.ndarray, ddof: int = 0,
     return float(var_val)
 
 
-def _estimate_ar_coefficient(EZZ_FB: np.ndarray, EZZ_BB: np.ndarray, 
+def _estimate_ar(EZZ_FB: np.ndarray, EZZ_BB: np.ndarray, 
                              vsmooth_sum: Optional[np.ndarray] = None,
                              T: Optional[int] = None) -> Tuple[np.ndarray, np.ndarray]:
     """Estimate AR coefficients and innovation variances from expectations."""
@@ -344,7 +355,7 @@ def _estimate_ar_coefficient(EZZ_FB: np.ndarray, EZZ_BB: np.ndarray,
     return A_diag, Q_diag
 
 
-def _clip_ar_coefficients(A: np.ndarray, min_val: float = -0.99, max_val: float = 0.99, 
+def _clip_ar(A: np.ndarray, min_val: float = -0.99, max_val: float = 0.99, 
                          warn: bool = True) -> Tuple[np.ndarray, Dict[str, Any]]:
     """Clip AR coefficients to stability bounds."""
     A_flat = A.flatten()
@@ -373,21 +384,20 @@ def _clip_ar_coefficients(A: np.ndarray, min_val: float = -0.99, max_val: float
 def _apply_ar_clipping(A: np.ndarray, config: Optional[Any] = None) -> Tuple[np.ndarray, Dict[str, Any]]:
     """Apply AR coefficient clipping based on configuration."""
     if config is None:
-        return _clip_ar_coefficients(A, -0.99, 0.99, True)
+        return _clip_ar(A, -0.99, 0.99, True)
     
-    from ..utils.helpers import safe_get_attr
     
-    clip_enabled = safe_get_attr(config, 'clip_ar_coefficients', True)
+    clip_enabled = getattr(config, 'clip_ar_coefficients', True) if config is not None else True
     if not clip_enabled:
         return A, {'n_clipped': 0, 'n_total': A.size, 'clipped_indices': []}
     
-    min_val = safe_get_attr(config, 'ar_clip_min', -0.99)
-    max_val = safe_get_attr(config, 'ar_clip_max', 0.99)
-    warn = safe_get_attr(config, 'warn_on_ar_clip', True)
-    return _clip_ar_coefficients(A, min_val, max_val, warn)
+    min_val = getattr(config, 'ar_clip_min', -0.99) if config is not None else -0.99
+    max_val = getattr(config, 'ar_clip_max', 0.99) if config is not None else 0.99
+    warn = getattr(config, 'warn_on_ar_clip', True) if config is not None else True
+    return _clip_ar(A, min_val, max_val, warn)
 
 
-def _compute_regularization_param(
+def _compute_reg_param(
     matrix: np.ndarray,
     scale_factor: float = 1e-5,
     warn: bool = True
@@ -438,7 +448,7 @@ def _compute_regularization_param(
     
     return reg_param, stats
 
-def _cap_max_eigenvalue(M: np.ndarray, max_eigenval: float = 1e6) -> np.ndarray:
+def _cap_eigenval(M: np.ndarray, max_eigenval: float = 1e6) -> np.ndarray:
     """Cap maximum eigenvalue of matrix to prevent numerical explosion.
     
     Parameters
@@ -474,7 +484,7 @@ def _cap_max_eigenvalue(M: np.ndarray, max_eigenval: float = 1e6) -> np.ndarray:
     return M
 
 
-def _ensure_innovation_variance_minimum(Q: np.ndarray, min_variance: float = 1e-8) -> np.ndarray:
+def _ensure_variance_min(Q: np.ndarray, min_variance: float = 1e-8) -> np.ndarray:
     """Ensure minimum variance on diagonal of innovation covariance matrix.
     
     Parameters
@@ -518,44 +528,6 @@ def _safe_divide(numerator: np.ndarray, denominator: np.ndarray, default: float
     with np.errstate(divide='ignore', invalid='ignore'):
         result = np.divide(numerator, denominator, out=np.full_like(numerator, default), where=denominator!=0)
     return result
-"""DDFM utility functions (training, state-space, conversion, loss).
-
-This module contains utility functions for DDFM extracted from ddfm.py
-to keep the main file under 1000 lines.
-"""
-
-import numpy as np
-from typing import Tuple, Optional, Any, TYPE_CHECKING
-import logging
-
-if TYPE_CHECKING:
-    import torch
-    import torch.nn as nn
-else:
-    torch = None
-    nn = None
-
-try:
-    import torch
-    import torch.nn as nn
-    import torch.optim as optim
-    _has_torch = True
-except ImportError:
-    _has_torch = False
-    if not TYPE_CHECKING:
-        torch = None
-        nn = None
-    optim = None
-
-from ..logger import get_logger
-
-_logger = get_logger(__name__)
-
-
-
-
-# State-space model building utilities
-
 
 
 
@@ -688,7 +660,7 @@ def estimate_var2(factors: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
     return A, Q
 
 
-def estimate_idiosyncratic_dynamics(
+def estimate_idio_dynamics(
     residuals: np.ndarray,
     missing_mask: np.ndarray,
     min_obs: int = 5,
@@ -903,16 +875,6 @@ def build_state_space(
     return A, Q, Z_0, V_0
 
 
-# extract_decoder_params is in ..core.vae
-from ..encoder.vae import extract_decoder_params
-
-
-
-
-# convert_decoder_to_numpy is in ..core.vae
-from ..encoder.vae import convert_decoder_to_numpy
-
-
 def estimate_state_space_params(
     f_t: np.ndarray,
     eps_t: np.ndarray,
@@ -932,7 +894,8 @@ def estimate_state_space_params(
     eps_t : np.ndarray
         Idiosyncratic terms (T x N)
     factor_order : int
-        Lag order for common factors (1 for VAR(1), 2 for VAR(2))
+        Lag order for common factors. Only VAR(1) and VAR(2) are supported.
+        Higher orders will raise NotImplementedError.
     bool_no_miss : np.ndarray, optional
         Boolean array (T x N) indicating non-missing values.
         If None, assumes no missing values.
@@ -998,7 +961,7 @@ def estimate_state_space_params(
         )
     
     # Estimate idiosyncratic AR(1) dynamics
-    A_eps, _, _ = estimate_idiosyncratic_params(eps_t, bool_no_miss, min_obs=5)
+    A_eps, _, _ = estimate_idio_params(eps_t, bool_no_miss, min_obs=5)
     
     # Construct companion form state vector and transition matrix
     if factor_order == 2:
@@ -1058,7 +1021,7 @@ def estimate_state_space_params(
     return A, W, mu_0, Σ_0, x_t
 
 
-def estimate_idiosyncratic_params(
+def estimate_idio_params(
     eps: np.ndarray,
     idx_no_missings: Optional[np.ndarray] = None,
     min_obs: int = 5,
@@ -1128,58 +1091,56 @@ def estimate_idiosyncratic_params(
     return phi, mu_eps, std_eps
 
 
-# Backward compatibility aliases
-get_transition_params = estimate_state_space_params
-get_idio = estimate_idiosyncratic_params
-
-
 def mse_missing(
-    y_actual: Any,  # torch.Tensor when torch is available
-    y_predicted: Any,  # torch.Tensor when torch is available
-) -> Any:  # torch.Tensor when torch is available
+    y_actual: Any,  # torch.Tensor or np.ndarray
+    y_predicted: Any,  # torch.Tensor or np.ndarray
+) -> Any:  # torch.Tensor or float
     """Mean Squared Error loss function that handles missing data.
     
     Computes MSE only on non-missing values. Missing values in y_actual
     (represented as NaN) are masked out from the loss computation.
     
+    Supports both PyTorch tensors and NumPy arrays automatically.
+    
     Parameters
     ----------
-    y_actual : torch.Tensor
-        Actual values (batch_size x N) with NaN for missing values
-    y_predicted : torch.Tensor
-        Predicted values (batch_size x N)
+    y_actual : torch.Tensor or np.ndarray
+        Actual values (batch_size x N or T x N) with NaN for missing values
+    y_predicted : torch.Tensor or np.ndarray
+        Predicted values (batch_size x N or T x N)
         
     Returns
     -------
-    torch.Tensor
+    torch.Tensor or float
         Scalar MSE loss computed only on non-missing values
-        
-    --------
+        Returns torch.Tensor for PyTorch inputs, float for NumPy inputs
     """
-    if not _has_torch:
-        raise ImportError("PyTorch is required for mse_missing")
-    
-    # Create mask: 1 for non-missing, 0 for missing
-    mask = torch.where(
-        torch.isnan(y_actual),
-        torch.zeros_like(y_actual),
-        torch.ones_like(y_actual)
-    )
-    
-    # Replace NaN with 0 for computation
-    y_actual_clean = torch.where(
-        torch.isnan(y_actual),
-        torch.zeros_like(y_actual),
-        y_actual
-    )
-    
-    # Apply mask to predictions
-    y_predicted_masked = y_predicted * mask
-    
-    # Compute MSE (automatically ignores masked values)
-    loss = nn.functional.mse_loss(y_actual_clean, y_predicted_masked, reduction='mean')
-    
-    return loss
+    # Auto-detect input type
+    if _has_torch and isinstance(y_actual, torch.Tensor):
+        # PyTorch path
+        # Create mask: 1 for non-missing, 0 for missing
+        mask = torch.where(
+            torch.isnan(y_actual),
+            torch.zeros_like(y_actual),
+            torch.ones_like(y_actual)
+        )
+        
+        # Replace NaN with 0 for computation
+        y_actual_clean = torch.where(
+            torch.isnan(y_actual),
+            torch.zeros_like(y_actual),
+            y_actual
+        )
+        
+        # Apply mask to predictions
+        y_predicted_masked = y_predicted * mask
+        
+        # Compute MSE (automatically ignores masked values)
+        loss = F.mse_loss(y_actual_clean, y_predicted_masked, reduction='mean')
+        return loss
+    else:
+        # NumPy path (fallback)
+        return mse_missing_numpy(y_actual, y_predicted)
 
 
 def convergence_checker(
@@ -1236,7 +1197,8 @@ def check_convergence(
     """Check convergence of reconstruction error.
     
     Computes the relative change in MSE between two iterations and checks
-    if convergence has been reached.
+    if convergence has been reached. This is a wrapper around convergence_checker()
+    that adds a converged flag.
     
     Parameters
     ----------
@@ -1257,33 +1219,11 @@ def check_convergence(
         Current MSE loss (on non-missing values)
     converged : bool
         True if relative change is below threshold
-        
-    --------
-    True
     """
-    # Mask for non-missing values
-    mask = ~np.isnan(y_actual)
-    
-    # Compute MSE on non-missing values (NumPy implementation, no sklearn dependency)
-    # Previous loss
-    y_prev_valid = y_prev[mask]
-    y_actual_valid = y_actual[mask]
-    loss_prev = np.mean((y_actual_valid - y_prev_valid) ** 2)
-    
-    # Current loss
-    y_now_valid = y_now[mask]
-    loss_now = np.mean((y_actual_valid - y_now_valid) ** 2)
-    
-    # Relative change
-    if loss_prev < 1e-10:
-        # Near-zero loss, consider converged
-        relative_change = 0.0
-    else:
-        relative_change = abs(loss_now - loss_prev) / loss_prev
-    
-    converged = relative_change < threshold
-    
-    return relative_change, loss_now, converged
+    # Use convergence_checker for the core computation
+    delta, loss_now = convergence_checker(y_prev, y_now, y_actual)
+    converged = delta < threshold
+    return delta, loss_now, converged
 
 
 def mse_missing_numpy(
